/* Layout */

/*
Navigation bar
*/

.navigation-menu {
    padding: 0;
    border-bottom: solid #FF7634 3px;
}

/*links in the navigation menu */
.navigation-menu .navbar-nav li > a {
    color: #0a8184;
}
.navigation-menu .navbar-nav li > a:hover {
    color: #1c56d0;
}

/*collapsed buttton */
.navigation-menu button[data-toggle] {
    background-color: white;
}

.navigation-menu button[data-toggle]:hover{
    background-color: #2c2c2c;
}

.nav > li > a.active {
    color: #ffffff;
    background-color: transparent;
}

.nav > li > a:hover {
    color: #ffffff;
    background-color: transparent;
    font-style: oblique;
}

.nav-tabs > li > a:hover {
    color: #337ab7;
}

.nav-login {
    float: right;
}

#username > a {
    color: #3d710b;
}

#username > a:hover {
    color: tomato;
}

#home_main {
    height: 45px;
}

/*
Warnings
*/

.messages_hide {
    display: none;
}

.messages_show {
    display: block;
}

#message_table_div {
    margin-top: 20px;
}


/*
Plot
*/

.disabled {
    pointer-events: none;  /* This makes it not clickable */
    opacity: 0.6;          /* This grays it out to look disabled */
}

.enabled {
    pointer-events: auto;  /* This makes it not clickable */
    opacity: 1;            /* This grays it out to look disabled */
}


/*
Footer
*/

footer {
    border-top: 0.1vw solid #E5E5E5;
    margin-top: 0vw;
    padding: 1.5vw 0 3.6vw;
}

.credits {
    background-color: #fff;
    padding: 50px 0;
}
.credits .sponsors {
    text-align: center;
}
.credits .sponsors a {
    display: inline-block;
    margin-right: 10px;
}

.credits .legal a {
    display: block;
    color: #000000;
}
.legal a:hover {
    color: #3e30f1;
}

/* home */
/* box with explanations */
.main-text-box {
    margin-top: 30px;
    margin-bottom: 30px;
    background-color: #fff;
    padding: 20px;
    border-radius: 10px;
    color: #6D6D6C;
    font-size: 1em;
    box-shadow: 3px 3px 5px #ddd;
    border: thin solid #FF7634;
}

.main-text-box h1,h2,h3 {
    font-weight: bold;
    font-family: "Lucinda Grande", "Lucinda Sans Unicode", Helvetica, Arial, Verdana, sans-serif;
    text-align: center;
}
.main-text-box p {
    font-family: "Droid Serif", "DejaVu Serif", "STIX", serif;
}

.-with-icon {
    display: inline
}

a.title-link {

}

/* panel body */
.panel-body
{
    overflow-y: auto;
}

.panel-transparent
{
    background: transparent;
}

.panel-background
{
    background: #efefef;
}

.panel .text-centered {
    text-align: center;
}

a.new_analysis {
    float: right;
    color: #2b542c;
}

.panel-examples {
    border-color: #FF7634;
}

.panel-examples>.panel-heading-examples {
    color: white;/*header text color*/
    background: #FF7634 none;
}
.panel-body .list-group .example_job {
    margin-bottom: 10px;
}

.example_job .example_job_heading a {
    color: #FF7634;
}

.example_job .list-group-item-text a.information {
    color: #337ab7;
}

.heading-basic-icon {
    font-weight: bold;
}
/* panel header row */
.panel-heading a.heading-basic-icon:before {
    font-family:'Glyphicons Halflings';
    content:"\e114";
    float: left;
    color: white;
}
.panel-heading a[class~=heading-basic-icon].collapsed:before {
    content:"\e080";
}

.example_job .links-to-actions a {
    color: #FF7634;
}
.example_job .links-to-actions a:hover {
    color: #337ab7;
}

.panel-basic {
    border-color: #337ab7;
}

.panel-basic>.panel-heading-basic {
    background: #337ab7 none;  /*not working */
    color: white;  /*header text color*/
}
.panel-body .list-group .user_job {
    margin-bottom: 10px;
}

.user_job .user_job_heading a{
    color: #337ab7;
}

.user_job .list-group-item-text a.information {
    color: #FF7634;
}

.user_job .links-to-actions a {
    color: #337ab7;
}
.user_job .links-to-actions a:hover {
    color: black;
}

.heading-examples-icon{
    font-weight: bold;
}

.panel-heading a.heading-examples-icon:before {
    font-family:'Glyphicons Halflings';
    content:"\e114";
    float: left;
    color: white;
}
.panel-heading a[class~=heading-examples-icon].collapsed:before {
    content:"\e080";
}

.panel-basic>.panel-heading-new_job {
    color: white;/*header text color*/
    background: #632aa4 none;
}

.heading-new_job-icon {
    font-weight: bold;
}

.panel-advanced {
    border-color: #337ab7;  /* #632aa4; */
    margin-bottom: 20px !important;
}

.panel-info {
    margin-bottom: 0px !important;
}

.panel-info .panel-body {
    font-size: 16px;
    line-height: 1.5;
}

.panel-advanced>.panel-heading-advanced {
    background: #337ab7 none;  /*not working, before: #632aa4 none; */
    color: white; /* header text color */
}

.heading-advanced-icon{
    font-weight: bold;
}

.panel-heading a.heading-advanced-icon:before {
    font-family: 'Glyphicons Halflings';
    content: "\e114";
    float: left;
    color: white;
}
.panel-heading a[class~=heading-advanced-icon].collapsed:before {
    content:"\e080";
}

.panel-advanced-options {
    border-color: black;
}

.panel-advanced>.panel-heading-advanced-options {
    /*background: #ff070f none;*/
    color: black;
}

.heading-advanced-options-icon{
    font-weight: bold;
}

.heading-submit-options-icon{
    font-weight: bold;
}

.panel-heading a.heading-advanced-options-icon:before {
    font-family:'Glyphicons Halflings';
    content:"\e114";
    float: left;
    color: black;
}

.panel-heading a.heading-submit-options-icon:before {
    font-family:'Glyphicons Halflings';
    content:"\e114";
    float: left;
    color: black;
}

.panel-heading a[class~=heading-submit-options-icon].collapsed:before {
    font-family:'Glyphicons Halflings';
    content:"\e080";
    float: left;
    color: black;
}

/* scroll bar on home panel */

.panel-body::-webkit-scrollbar-track {
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.3);
	border-radius: 10px;
	background-color: transparent;/*#F5F5F5;*/
}
.panel-body::-webkit-scrollbar {
	width: 12px;
	background-color: transparent;/*#F5F5F5;*/
}
.panel-body::-webkit-scrollbar-thumb {
	border-radius: 10px;
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,.3);
	background-color: transparent/*#9d9d9d*/;
}

.popover {
    max-width: 100%; /* Max Width of the popover (depending on the container!)*/
}

body {
    /* background:#F8F8F8 url("../images/background.jpg") center bottom 0px repeat-x; */
    font-family:'Open Sans', sans-serif;
}

/* login */
.login-button {
    text-align: center;
}

.login-page-size {
    height: 200px;
}

/*analysis task */
.panel-job_details {
    border-color: #FF7634;
    background-color: #EBEBEB;
}
/*
.panel-job_details>.panel-heading-job {
    color: white;
    background: #ff070f none;
}
*/
.heading-job_details-icon {
    font-weight: bold;
}

.heading-job_details-icon span {
    color: #FF7634;
    float: right;
    font-size: .8em;
    font-weight: normal;
}


.panel-job_details .actions {
    margin: 20px auto 10px;
    clear: both;

}
.panel-job_details .actions  .btn-default{
    border: none;
    box-shadow: 1px 1px 5px RGBA(0, 0, 0, .1);
    margin-top:3px;
}

/*
Terminal
*/
.terminal {
    height: 340px;
    outline: none;
    line-height: 1em;
    font-size: 14px;
    float: left;
    font-family: monospace;
    color: white;
    background: black;
    padding: .4em;
    border-radius: 2px;
    -webkit-box-shadow: 0 0 12px 1px rgba(87,87,87,0.4);
    box-shadow: 0 0 12px 1px rgba(87,87,87,0.4);
    overflow-y:scroll;
    overflow-x:hidden;
    margin-top: 0px;
    width: 100%;
}

iframe.for_bokeh {
    width: 100%;
    height: 1000px;
    border: 0;
}

input[type='range']{
    -moz-appearance: none;
    -webkit-appearance: none;
    height: 3px;
    background-color: #ddd;
}

input[type='range']::-webkit-slider-thumb {
    -webkit-appearance: none;
    background-color: black;
    width: 20px;
    height: 20px;
    border-radius: 10px;
    border: 1px solid #bdc3c7;
    cursor: pointer;

}

input[type='range']::-moz-range-thumb {
    -moz-appearance: none;
    background-color: black;
    width: 20px;
    height: 20px;
    border-radius: 10px;
    border: 1px solid #bdc3c7;
    cursor: pointer;
}

.sliderVal {
    position: relative;
    display: inline-block;
}

.sliderVal .sliderValText {
    /*visibility: hidden;*/
    width: 80px;
    background-color: black;
    color: #fff;
    text-align: center;
    border-radius: 6px;
    padding: 5px 0;
    position: absolute;
    z-index: 1;
    top: 150%;
    left: 50%;
    margin-left: -60px;
}

/* text arrow */
.sliderVal .sliderValText::after {
    content: "";
    position: absolute;
    bottom: 100%;
    left: 50%;
    margin-left: -5px;
    border: 5px solid transparent;
    border-bottom-color: black;
}

.sliderVal:hover .sliderValText {
    /*visibility: visible;*/
}

.sharing input {

}
/*
ul.email-list {
    max-height:250px;
    overflow:hidden;
    overflow-y:scroll;
}*/
div.email-list {
    max-height:250px;
    overflow:hidden;
    overflow-y:scroll;
}


/* scroll bar on email list */

.email-list::-webkit-scrollbar-track {
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.3);
	border-radius: 10px;
	background-color: transparent;/*#F5F5F5;*/
}
.email-list::-webkit-scrollbar {
	width: 12px;
	background-color: transparent;/*#F5F5F5;*/
}
.email-list::-webkit-scrollbar-thumb {
	border-radius: 10px;
	-webkit-box-shadow: inset 0 0 6px rgba(0,0,0,.3);
	background-color: transparent/*#9d9d9d*/;
}

.pre {
    white-space: pre-line;
    text-align: justify;
    background-color: #eee;
    padding: 50px 50px;
}


/* data */
#data_alert_header {
    font-size: 21px;
    font-weight: 200;
}

#data_alert_footer {
    font-size: 21px;
    font-weight: 200;
}

.underline {
    text-decoration: underline;
}