{% block jumbotron %}
        <!-- Main jumbotron -->
        <div class="jumbotron">
            <h1>The Islet Regulome Browser</h1>
            <p>The Islet Regulome Browser is a visualization tool that provides access to interactive exploration of pancreatic islet genomic data.</p>
        </div>
{% endblock %}
