{% extends "base.html" %}
{% block body %}
        <!-- Form -->
        <form name="form_base" id="form_base" class="form-horizontal" role="form" action="submit" enctype="multipart/form-data" method="POST">
            <input type="hidden" id="navbar_link" name="navbar_link">
        </form>
        <!-- Jumbotron -->
        <div class="jumbotron">
            <h1>The Islet Regulome Browser</h1>
            <p>
                The Islet Regulome browser was designed by Dr. <span class="bold">Lorenzo Pasquali</span>
                (IGTP/IJC) and Dr. <span class="bold">Loris Mularoni</span> (IRB) who implemented and developed
                both the code for generating plots and the web interface. Initial supervision for the project
                was provided by Prof. Jorge Ferrer (ICL) at Institut D'Investigacions Biomediques August Pi
                i Sunyer (IDIBAPS).
            </p>
            <p>
                The Islet Regulome Browser is now developed and maintained in the <span class="underline">
                <a href="http://www.endoregulatorygenomics.org/">Pasquali Lab</a></span> by <span class="underline">
                Mireia Ramos</span> and <span class="underline">Dr. Loris Mularoni</span> with support from the
                Program for Predictive and Personalized Medicine of Cancer (PMPPC) at the Institute Germans Trias i
                Pujol (IGTP) Bioinformatics Core.
            </p>
        </div>

{% endblock %}
