{% extends "base.html" %}
{% block body %}
        <!-- Form -->
        <form name="form_base" id="form_base" class="form-horizontal" role="form" action="submit" enctype="multipart/form-data" method="POST">
            <input type="hidden" id="navbar_link" name="navbar_link">
        </form>
        <!-- Jumbotron -->
        <div class="jumbotron">
            <h1>The Islet Regulome Browser</h1>
            <p>
                We encourage you to contact us to ask questions, leave suggestions, comments, feedback, and any other concerns. For bug reports,
                please include the type and version of the browser you are using along with the description of the bug.
            </p>
            <p>
                <a class="btn btn-primary btn-lg" href="mailto:info@isletregulome.com" role="button">Contact Us</a>
            </p>
        </div>
{% endblock %}
