body {
	background-color: #BBBBBB; /* #58ACFA; */
	text-align: center; /* for IE */
	margin: 0 auto; /* for the rest */
	font-family: Verdana, Geneva, Arial, sans-serif;
	font-size: medium;
}

#contentmessages {
	position: relative;
	margin-left: auto;
	margin-right: auto;
	margin-top: 9px;
	margin-bottom: 9px;
	width: 920px;
	height: 30px;
	padding: 10px;
	border-radius: 20px;
	color: #DDDDDD;
	font-weight: bold;
	text-align: center;
	vertical-align: middle;
	line-height: 30px;
}

.messages_show {
	border: 1px solid #c0392b;
	background-color: #e74c3c;
}

.messages_hide {
	border: 1px solid #BBBBBB;
	background-color: #BBBBBB;
}

.close {
    float: right;
    color: #DDDDDD;
    background-color: #c0392b;
    width: 18px;
    height: 18px;
    border-radius: 9px;
    text-align: center;
	vertical-align: middle;
	line-height: 15px;
    cursor: pointer;
}

div.messages_hide > div.close {
    display: none;
}

#contentwrapper {
	position: relative;
	margin-left: auto;
	margin-right: auto;
	margin-top: 0px;
	margin-bottom: 50px;
	width: 920px;
	padding: 10px;
	border: 1px solid #3182BD;
	background-color: #9ECAE1;
	border-radius: 20px;
}

#header {
	background-color: #3182BD; 
	border: 1px solid #3182BD; 
	border-radius: 20px;
	color: #BBBBBB;
	margin-bottom: 10px;
}

#logout {
	position: absolute;
	top: 60px;
	right: 30px;
	font-size: small;
}

#version {
    position: absolute;
    top: 60px;
    left: 30px;
    font-size: small;
}

#build {
	
}

#info_icon_1 {
	position: absolute;
	top: 78px;
	left: 390px;
}

#info_icon_2 {
	position: absolute;
	top: 262px;
	left: 390px;
}

#doStuff {} 

#doLogin {
	background-color: #DEEBF7;
	margin: 0px 0px 0px 0px;
	padding-top: 5px;
	padding-bottom: 20px;
	border-radius: 20px;
}

#doUpload {
	position: relative;
	float: right;
	background-color: #DEEBF7;
	width: 450px;
	height: 500px;
	padding-top: 5px;
	border-radius: 20px;
}

#doSignup {
    position: relative;
    float: right;
    background-color: #DEEBF7;
    width: 450px;
    height: 300px;
    padding-top: 30px;
    border-radius: 20px;
}

#doSignin {
    position: relative;
    background-color: #DEEBF7;
    height: 300px;
    margin: 0px 460px 0px 0px;
    padding-top: 30px;
    border-radius: 20px;
}

#gene_name {
	width: 250px;
}

.buttons {
	position: absolute;
	bottom: 10px;
	right: 0; 
	left: 0; 
	margin-left: auto; 
	margin-right: auto;
	width: 100px;
	height: 30px;
	color: #3182BD;
}

.buttons_dx {
    margin-right: 80px;
}

.buttons_sx {
	margin-left: 80px;
}

input[type="submit"] {
	cursor: pointer;
	-webkit-appearance: button;
}

#doPlot {
	position: relative;
	background-color: #DEEBF7; 
	height: 500px;
	margin: 0px 460px 0px 0px;
	padding-top: 5px;
	border-radius: 20px;
}

#footer {
	height: 100px;
	margin-top: 10px;
	border: 1px solid #3182BD;
	background-color: #3182BD;
	border-radius: 20px;
	color: #BBBBBB;
}

.fieldset {
	width: 400px;
	text-align: center;
	margin: auto;
	overflow: auto;
	margin-bottom: 4px;
	border: 2px solid #3182BD;
	padding: 10px;
	border-radius: 20px;
}

.legend {
	position: absolute;
	font-weight: bold;
	padding: 0.2em 0.5em;
	color: black;
	background-color: #DEEBF7;
	line-height: 10px;
	z-index: 3;
}

.ltop { top: 95px; }
.lbottom { top: 918px; }

#legend_top_left { margin-left: 60px; }
#legend_top_right { margin-left: 50px; }
#legend_top_middle { margin-left: 190px; }

#legend_bottom_left { margin-left: 53px; }
#legend_bottom_right { margin-left: 45px; }
#legend_bottom_middle { margin-left: 180px; }

#legend_choose_gene { top: 132px; margin-left: 250px; background-color: #9ECAE1; }

#img_plot {
	width: 894px;
	height: 734px;
	border-radius: 20px;
	/* This subsitute: border: 2px solid #3182BD */
	-webkit-box-shadow: 0px 0px 0px 2px #3182BD; /* Saf3-4, iOS 4.0.2 - 4.2, Android 2.3+ */
	box-shadow: 0px 0px 0px 2px #3182BD; /* Opera 10.5, IE9, FF4+, Chrome 6+, iOS 5 */
	overflow: hidden;
}

#img_egg {
	width: 449px;
	/*height: 734px; */
	border-radius: 20px;
	/* This subsitute: border: 2px solid #3182BD */
	-webkit-box-shadow: 0px 0px 0px 2px #3182BD; /* Saf3-4, iOS 4.0.2 - 4.2, Android 2.3+ */
	box-shadow: 0px 0px 0px 2px #3182BD; /* Opera 10.5, IE9, FF4+, Chrome 6+, iOS 5 */
	overflow: hidden;
	margin-bottom: 25px;
}

html.ie7 img, html.ie8 img { border:2px solid #3182BD; } 

.info_icon {
	position: absolute;
	right: 0px;
	left: 0px;
	margin-left: auto;
	cursor: pointer;
}

.info_2 {
	top: 223px;
	margin-right: 110px;
}

.info_3 {
	top: 230px;
	margin-right: 100px;	
}

#main_plot {
	background-color: #DEEBF7;
	width: 920px;
	margin: 0px 0px 10px 0px;
	padding: 0px 0px 20px 0px;
	border-radius: 20px;
}

#main_table {
	background-color: #DEEBF7;
	width: 920px;
	margin: 0px 0px 0px 0px;
	padding: 20px 0px 20px 0px;
	border-radius: 20px;
}

#motif_table {
	background-color: #DEEBF7;
	width: 920px;
	margin: 0px 0px 0px 0px;
	padding: 20px 0px 20px 0px;
	margin-bottom: 10px;
	border-radius: 20px;
}

.top_controls {
	width: 200px;
	height: 60px;
	border-radius: 20px;
	border: 2px solid #3182BD;
	float: left;
	margin-top: 10px;
	margin-bottom: 12px;
	margin-left: 10px;
	text-align: center;
	overflow: auto;
	padding: 0px;
}

#zoom_in {}
#zoom_out {}

#region_control { width: 468px; /* 920 -20 -12 -20 */ }
#region_control input { width: 60px; }

#move_left { margin-top: 8px; }
#move_right { margin-top: 8px; }

#download {
	margin-top: 8px;
	width: 468px; /* 920 -20 -12 -20 */
}

#another_plot { padding-top: 10px; clear: both;}

table.resultable {
	text-align: center; /* for IE */
	margin: 0 auto; /* for the rest */
	width: 650px;
	font-family: verdana,arial,sans-serif;
	font-size: 12px;
	border-collapse: separate; /* collapse; */
	padding: 8px;
	border-spacing: 0;
}

table.resultable tr { height:20px; }

table.resultable tr th, table.resultable tr td {
	border-right: 1px solid #bbb;
	border-bottom: 1px solid #bbb;
	padding: 8px;
	font-size: 12px;
}

table.resultable tr th {
	background: #dedede;
	border-top: 2px solid #3182BD; //1px solid #bbb;
}

table.resultable tr th:first-child, table.resultable tr td:first-child {
	border-left: 2px solid #3182BD;
}

table.resultable tr th:last-child, table.resultable tr td:last-child {
	border-right: 2px solid #3182BD;
}

table.resultable tr:first-child th:first-child {
	border-top-left-radius: 10px;
}

table.resultable tr:first-child th:last-child {
	border-top-right-radius: 10px;
}

table.resultable tr:last-child td {
	border-bottom: 2px solid #3182BD; 
}

table.resultable tr:last-child td:first-child {
	border-bottom-left-radius: 10px;
}

table.resultable tr:last-child td:last-child {
	border-bottom-right-radius: 10px;
}

#citation { text-decoration: underline; }
#authors { font-style: italic; }
#journal { font-style: italic; }

#credits { 
	position: absolute; 
	right: 440px;
	left: 440px;
	margin-top: -5px; 
	margin-right: auto; 
	margin-left: auto;
}

#home { float: left; margin-left: 100px; margin-top: -5px;}
#contact { float: right; margin-right: 100px; margin-top: -5px;}

a:link { color: #bbb; text-decoration: none;}
a:visited { color: #bbb; text-decoration: none;}
a:hover { color: #DEEBF7; text-decoration: underline;}

#error a:link { color: #3182BD; }

.frame {
	text-align: left;
	background-color: #DEEBF7;
	width: 880px;
	border-radius: 20px;
	font-size: small;
}

.frame h3 { text-align: center; margin: auto;}

#frame_up {
	margin: 0px 0px 10px 0px;
	padding: 20px;
}

#frame_middle {
	margin: 0px 0px 10px 0px;
	padding: 20px;
}

#frame_down {
	margin: 0px 0px 0px 0px;
	padding: 20px;
}

.bold { font-weight: bold; }

#frame_up a:link { color: #3182BD; }
#frame_up a:visited { color: #3182BD; }
#frame_up a:hover { color: #3182BD; }
#frame_down a:link { color: #3182BD; }
#frame_down a:visited { color: #3182BD; }
#frame_down a:hover { color: #3182BD; }

.hiddenRow { display: none; }

#loader {
	//display: none;
	position: absolute; 
	bottom: 100px;
	right: 0;
	left: 0;
	margin-left: auto;
	margin-right: auto;
}

#loader_caption {
	//display: none;
	position: absolute;
	bottom: 130px;
	right: 0;
	left: 0;
	margin-left: auto;
	margin-right: auto;
}

/* Style for the db page*/
#index_db {
	width: 150px;
	text-align: left;
	margin-left:auto; 
	margin-right:auto;
	margin-bottom: 100px;
}

#index_db a:link { color: #3182BD; }
#index_db a:visited { color: #3182BD; text-decoration: none;}
#index_db a:hover { color: #DEEBF7; text-decoration: underline;}

