{% extends "base.html" %}
{% block body %}
        {% include 'jumbotron_header.html' %}

        <script type="text/python" src="{{ url_for('static', filename='python/data.py') }}"></script>

        <!-- Form -->
        <form name="form_base" id="form_base" class="form-horizontal" role="form" action="submit" enctype="multipart/form-data" method="POST">
            <input type="hidden" id="navbar_link" name="navbar_link">

            <!-- Header alert -->
            <div id="data_alert_header" class="alert alert-danger alert-dismissible" role="alert">
                <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                <p>The following links contain all the raw data currently available in the Islet Regulome Browser</p>
            </div>

            <!-- Accordion -->
            <div class="panel-group" id="accordion">

                <!-- Adult human islets -->
                <div class="panel panel-primary panel-background panel-basic panel-data">
                    <!-- Header -->
                    <div class="panel-heading panel-heading-basic text-centered">
                        <h4 class="panel-title -with-icon">
                            <a id="data_human_islet_icon" class="heading-basic-icon" data-toggle="collapse" data-parent="#accordion" href="#data_human_islet">Adult human islets</a>
                        </h4>
                    </div>
                    <!-- Body -->
                    <div id="data_human_islet" class="panel-collapse collapse in">
                        <div class="panel-body">
                            <div class="panel-group" id="form_panel_adult">
                                <!-- Open chromatin classes / enhancer clusters / transcription factors-->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_pancreatic_islet_enhancers_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_adult" href="#data_pancreatic_islet_enhancers"> Open chromatin classes / enhancer clusters / transcription factors</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_pancreatic_islet_enhancers" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <div class="form-group">
                                                <p>
                                                    Pancreatic islet enhancer clusters enriched in type 2 diabetes risk-associated
                                                    variants. Pasquali L, et al. Nat Genet. 2014 Feb;46(2):136-43. doi: 10.1038/ng.2870.
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li><a href="http://www.nature.com/ng/journal/v46/n2/source_data/ng.2870-f2.xlsx">Open chromatin classes</a></li>
                                                        <li><a href="http://www.nature.com/ng/journal/v46/n2/source_data/ng.2870-sf6.xlsx">Enhancer Clusters</a></li>
                                                        <li>H3K27ac, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>H3K4me1, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>PDX1, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>NKX2.2, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>NKX6.1, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>MAFB, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>FOXA2, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                        <li>CTCF, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1919/samples/?s_page=1&s_pagesize=50">E-MTAB-1919</a></li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>

                                <!-- Pancreatic islet transcriptome -->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_human_b_cell_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_adult" href="#data_human_b_cell"> Pancreatic islet transcriptome</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_human_b_cell" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <!-- datasets -->
                                            <div class="form-group">
                                                <p>
                                                    Human β cell transcriptome analysis uncovers lncRNAs that are tissue-specific,
                                                    dynamically regulated, and abnormally expressed in type 2 diabetes. Morán I, et
                                                    al. Cell Metab. 2012 Oct 3;16(4):435-48. doi: 10.1016/j.cmet.2012.08.010.
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li>H3K4me3, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1294/">E-MTAB-1294</a></li>
                                                        <li>RNA-seq, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1294/">E-MTAB-1294</a></li>
                                                        <li>lncRNA (from <a href="http://www.cell.com/cms/attachment/2007962486/2030684089/mmc3.xls">Supplementary Table 4</a>)</li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>

                                <!-- Pancreatic islet open chromatin map -->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_open_chromatin_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_adult" href="#data_open_chromatin"> Pancreatic islet open chromatin map</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_open_chromatin" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <div class="form-group">
                                                <p>
                                                    A map of open chromatin in human pancreatic islets. Gaulton KJ, et al. Nat Genet. 2010
                                                    Mar;42(3):255-9. doi: 10.1038/ng.530.
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li>FAIRE-seq, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-GEOD-17616/">E-GEOD-17616</a></li>
                                                        <li><a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-3199/">E-MTAB-3199</a>, Appendix to E-GEOD-17616</li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>

                                <!-- Chromatin states / stretch enhancers -->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_chromatin_state_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_adult" href="#data_chromatin_state"> Chromatin states / stretch enhancers</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_chromatin_state" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <div class="form-group">
                                                <p>
                                                    Chromatin stretch enhancer states drive cell-specific gene regulation and harbor
                                                    human disease risk variants. Parker SC, et al. Proc Natl Acad Sci U S A. 2013 Oct
                                                    29;110(44):17921-6. doi: 10.1073/pnas.1317023110.
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li><a href="http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE51311">Chromatin states</a></li>
                                                        <li><a href="http://research.nhgri.nih.gov/manuscripts/Collins/islet_chromatin/hg19/ChromHMM/stretch-enhancers.tar">Stretch enhancers</a></li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>

                                <!-- Alpha and beta cell open chromatin map -->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_human_alpha_cell_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_adult" href="#data_human_alpha_cell"> Alpha and beta cell open chromatin map</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_human_alpha_cell" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <div class="form-group">
                                                <p>
                                                    Integration of ATAC-seq and RNA-seq Identifies Human Alpha Cell and Beta Cell
                                                    Signature Genes. Ackerman AM, et al. Mol Metab 2016 Mar;5(3):233-44. doi:
                                                    10.1016/j.molmet.2016.01.002
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li><a href="http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE76268">ATAC-seq of FACS purified alpha and beta cells</a></li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>

                <!-- Progenitors -->
                <div class="panel panel-primary panel-background panel-basic panel-data">
                    <!-- Header -->
                    <div class="panel-heading panel-heading-basic text-centered">
                        <h4 class="panel-title -with-icon">
                            <a id="data_progenitor_icon" class="heading-basic-icon collapsed" data-toggle="collapse" data-parent="#accordion" href="#data_progenitor">Pancreatic progenitors</a>
                        </h4>
                    </div>
                    <!-- Body -->
                    <div id="data_progenitor" class="panel-collapse collapse">
                        <div class="panel-body">
                            <div class="panel-group" id="form_panel_progenitors">
                                <!-- In vitro pancreatic multipotent progenitor cells (MPCs)-->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_invitro_progenitors_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_progenitors" href="#data_invitro_progenitors"> Pancreatic progenitors regulatory regions / transcription factors</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_invitro_progenitors" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <!-- datasets -->
                                            <div class="form-group">
                                                <p>
                                                    TEAD and YAP regulate the enhancer network of human embryonic pancreatic progenitors.
                                                    Cebola I, et al. Nat Cell Biol. 2015 May;17(5):615-26. doi: 10.1038/ncb3160
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li><a href="http://www.ncbi.nlm.nih.gov/pmc/articles/PMC4434585/bin/NIHMS62644-supplement-Supp_Tables.xlsx">MPC enhanceres, (from Supplementary figure 6)</a></li>
                                                        <li>GATA6, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1990">E-MTAB-1990</a></li>
                                                        <li>ONECUT1, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1990">E-MTAB-1990</a></li>
                                                        <li>FOXA2, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1990">E-MTAB-1990</a></li>
                                                        <li>PDX1, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1990">E-MTAB-1990</a></li>
                                                        <li>HNF1B, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-1990">E-MTAB-1990</a></li>
                                                        <li>TEAD1, <a href="http://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-3061/">TE-MTAB-3061</a></li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>

                <!-- GWAS variants -->
                <div class="panel panel-primary panel-background panel-basic panel-data">
                    <!-- Header -->
                    <div class="panel-heading panel-heading-basic text-centered">
                        <h4 class="panel-title -with-icon">
                            <a id="data_gwas_icon" class="heading-basic-icon collapsed" data-toggle="collapse" data-parent="#accordion" href="#data_gwas">GWAS variants</a>
                        </h4>
                    </div>
                    <!-- Body -->
                    <div id="data_gwas" class="panel-collapse collapse">
                        <div class="panel-body">
                            <div class="panel-group" id="form_panel_gwas">
                                <!-- Diagram-->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_diagram_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_gwas" href="#data_diagram"> T2D - DIAGRAM</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_diagram" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <div class="form-group">
                                                <p>
                                                    Large-scale association analysis provides insights into the genetic architecture and pathophysiology of type 2 diabetes.
                                                    Morris AP, et al. Nat Genet. 2012 Sep;44(9):981-90. doi: 10.1038/ng.2383.
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li><a href="http://www.ncbi.nlm.nih.gov/pubmed/22885922">DIAGRAM</a></li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>

                                <!-- Magic-->
                                <div class="panel panel-default panel-submit-options">
                                    <!-- Header -->
                                    <div class="panel-heading panel-heading-submit-options">  <!-- -->
                                        <h4 class="panel-title ">
                                            <a id="data_magic_icon" class="heading-submit-options-icon collapsed" data-toggle="collapse" data-parent="#form_panel_gwas" href="#data_magic"> Fasting glycemia - MAGIC</a>
                                        </h4>
                                    </div>
                                    <!-- Body -->
                                    <div id="data_magic" class="panel-collapse collapse">  <!-- -->
                                        <div class="panel-body">
                                            <div class="form-group">
                                                <p>
                                                    Large-scale association analyses identify new loci influencing glycemic traits and provide insight into the underlying biological pathways.
                                                    Scott RA, Nat Genet. 2012 Sep;44(9):991-1005. doi: 10.1038/ng.2385.
                                                </p>
                                                <div>
                                                    <ul>
                                                        <li><a href="http://www.ncbi.nlm.nih.gov/pubmed/22885924">MAGIC</a></li>
                                                    </ul>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>

            </div>

            <!-- Footer alert -->
            <div id="data_alert_footer" class="alert alert-danger alert-dismissible" role="alert">
                <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                <!--p>Please <a href="mailto:info@isletregulome.com">contact us</a> to suggest new islet datasets for permanent display in the Islet Regulome Browser</p-->
                <p>Please <a href="{{ url_for('contact') }}">contact us</a> to suggest new islet datasets for permanent display in the Islet Regulome Browser</p>
            </div>

        </form>
{% endblock %}

