{% extends "base.html" %}
{% block body %}
    <h3>Not implemented yet!</h3>
    <table style="text-align:center;margin:0 auto;border:solid 1px;">
    {%- for key, value in messages.items() %}
        <tr>
            <td>{{ key }}</td>
            <td>{{ value }}</td>
        </tr>
    {% endfor -%}
    </table>
    {{ dataframe | safe }}
{% endblock %}
