from browser import html, document
from .dialog import *
from .progressbar import *
from .slider import *
from .menu import *

def add_stylesheet():
    # should use urllib.parse, but it imports so many modules...
    css_url = '/'.join(__file__.split('/')[:-1]+
        ['css/smoothness/jquery-ui-1.10.3.custom.min.css'])
    _link=html.LINK(Href=css_url)
    _link.rel='stylesheet'

    document <= _link
